unit SquareGrid;

interface

uses
    Windows, SysUtils, Classes, Graphics;

//=============================================================================
type
    TSquareGrid = class

    //////
    public
        //  RXgN^
        constructor Create();
        //  fXgN^
        destructor Destroy(); override;

    //////
    private
        //  Z̈ӂ̒
        FCellSize: Integer;
        //  ̕
        FLineSize: Integer;
        //  ̃Z
        FColCount: Integer;
        //  c̃Z
        FRowCount: Integer;
        //  ̔p}[W
        FMargin: Integer;
        //  Ŝ\̂ɕKvȉ(vZl)
        FWidth: Integer;
        //  Ŝ\̂ɕKvȏc(vZl)
        FHeight: Integer;
        //  Zbg֐
        procedure SetCellSize(const AValue: Integer);
        procedure SetLineSize(const AValue: Integer);
        procedure SetColCount(const AValue: Integer);
        procedure SetRowCount(const AValue: Integer);
        //  ČvZ
        procedure UpdateProperty();
    public
        //  ڌJϐ
        LineColor: TColor;
        CellColor: TColor;

        //  vpeB
        property CellSize: Integer read FCellSize write SetCellSize;
        property LineSize: Integer read FLineSize write SetLineSize;
        property ColCount: Integer read FColCount write SetColCount;
        property RowCount: Integer read FRowCount write SetRowCount;
        property Width: Integer read FWidth;
        property Height: Integer read FHeight;
    //////
    public
        //  Obh`
        procedure DrawGrid(const AContext: HDC; const ALeft, ATop: Integer);
        //  Zʒu擾 (0,0)
        function Cell(const X, Y: Integer): TRect;
        //  NbNʒuZʒuɕϊ
        function MouseToCell(const MouseX, MouseY: Integer): TPoint;
    //////
    end;

var
    Square: TSquareGrid = nil;

implementation

//=============================================================================

//////
//-----------------------------------------------------------------------------
//  RXgN^
constructor TSquareGrid.Create();
begin
    FCellSize := 24;
    FLineSize := 2;
    FColCount := 8;
    FRowCount := 8;

    LineColor := clBlack;
    CellColor := clWhite;

    UpdateProperty();
end;

//-----------------------------------------------------------------------------
//  fXgN^
destructor TSquareGrid.Destroy();
begin
    inherited;
end;

//////
//-----------------------------------------------------------------------------
//  Zbg֐
procedure TSquareGrid.SetCellSize(const AValue: Integer);
begin
    if (AValue >0) then
    begin
        FCellSize := AValue;
        UpdateProperty();
    end;
end;

//-----------------------------------------------------------------------------
procedure TSquareGrid.SetLineSize(const AValue: Integer);
begin
    if (AValue >0) then
    begin
        FLineSize := AValue;
        UpdateProperty();
    end;
end;

//-----------------------------------------------------------------------------
procedure TSquareGrid.SetColCount(const AValue: Integer);
begin
    if (AValue >0) then
    begin
        FColCount := AValue;
        UpdateProperty();
    end;
end;

//-----------------------------------------------------------------------------
procedure TSquareGrid.SetRowCount(const AValue: Integer);
begin
    if (AValue >0) then
    begin
        FRowCount := AValue;
        UpdateProperty();
    end;
end;

//-----------------------------------------------------------------------------
//  ČvZ
procedure TSquareGrid.UpdateProperty();
begin
    //  }[WvZ
    FMargin := FLineSize div 2;
    if (Odd(FLineSize)) then
    begin
        //Inc(FMargin);
    end;
    //  S̕Zo
    FWidth := (FCellSize + FLineSize) * FColCount + FLineSize;
    //  S̍Zo
    FHeight := (FCellSize + FLineSize) * FRowCount + FLineSize;
end;

//////
//-----------------------------------------------------------------------------
//  Obh`
procedure TSquareGrid.DrawGrid(const AContext: HDC; const ALeft, ATop: Integer);
var
    LCanvas: TCanvas;
    i: Integer;
    LX, LY: Integer;
    LBackPen: TColor;
    LBackBrush: TColor;
begin
    LCanvas := nil;
    try
        LCanvas := TCanvas.Create();
        LCanvas.Handle := AContext;

        LBackPen := LCanvas.Pen.Color;
        LBackBrush := LCanvas.Pen.Color;

        LCanvas.Pen.Color := LineColor;
        LCanvas.Brush.Color := CellColor;
        LCanvas.Pen.Width := LineSize;

        LX := {FMargin + }ALeft;
        LY := FMargin + ATop;
        for i := 1 to FRowCount + 1 do
        begin
            LCanvas.MoveTo(LX, LY);
            LCanvas.LineTo(LX + FWidth, LY);
            Inc(LY, FCellSize + FLineSize);
        end;

        LX := FMargin + ALeft;
        LY := FMargin + ATop;
        for i := 1 to FColCount + 1 do
        begin
            LCanvas.MoveTo(LX, LY);
            LCanvas.LineTo(LX, LY + FHeight);
            Inc(LX, FCellSize + FLineSize);
        end;

        LCanvas.Pen.Color := LBackPen;
        LCanvas.Pen.Color := LBackBrush;
    finally
        LCanvas.Free();
    end;
end;

//-----------------------------------------------------------------------------
//  Zʒu擾 (0,0)
function TSquareGrid.Cell(const X, Y: Integer): TRect;
begin
    Result.Left   := (FLineSize + FCellSize) * X + FLineSize;
    Result.Top    := (FLineSize + FCellSize) * Y + FLineSize;
    Result.Right  := Result.Left + FCellSize;
    Result.Bottom := Result.Top + FCellSize;
end;

//-----------------------------------------------------------------------------
//  NbNʒuZʒuɕϊ
function TSquareGrid.MouseToCell(const MouseX, MouseY: Integer): TPoint;
begin
    Result.X := -1;
    Result.Y := -1;

    //  ͂ݏo`FbN
    if (0 <= MouseX) and (MouseX < FWidth)
        and (0 <= MouseY) or (MouseY < FHeight) then
    begin
        Result.X := (MouseX div (FCellSize + FLineSize));
        Result.Y := (MouseY div (FCellSize + FLineSize));

        //  E[̃`FbN~Xɉ
        if (FColCount <= Result.X) then Dec(Result.X);
        if (FRowCount <= Result.Y) then Dec(Result.Y);
    end;
end;


//////
initialization
    Square := TSquareGrid.Create();
finalization
    Square.Free();
end.

